/******************************************************************************

ES_DesolokChild
-------------------------------------------------------------------------------
Child of a Desolok, created when a Desolok in the hive is large enough,
walks around a bit and then turns in a real Desolok.
-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Jul 7, 2002
Version:    1.0

******************************************************************************/

package EcoSystem;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.*;


public class ES_DesolokChild extends ES_Animal
{
  final static int BNI_GROWBEHAVIOR       = 2;

  final static int STATE_WANDER           = 0;
  final static int STATE_ENROUTETOPLACE   = 1;
  final static int STATE_SEARCHFOOD       = 2;
  final static int STATE_ENROUTETOFOOD    = 3;
  final static int STATE_EATFOOD          = 4;

  final static float DECIDE_EAT           = 2.5f;

  final static float CRITICAL_EAT         = 30.0f;

  final static float  MOVESPEED           = 0.35f;
  final static float  LOOKSPEED           = 100.0f;
  final static double LOOKANGLE           = Math.PI / 10;
  final static double LOOKDISTANCE        = 1.0f;

  //                                        .5 minute as a child is enough!
  final static long     CHILDTIME         = 30*1000;

  final protected long  timeborn          = ES_Util.GetESTime();

  protected float       hunger            = 0.0f;

  protected Transform3D   lastKnownFlowerPos  = null;

  protected ES_LifeForm wantToEat   = null;

  protected int curState          = 0;

  protected ES_Desolok myMother = null;

  protected TransformGroup    myGrowTG           = null;
  protected Alpha             myGrowAlpha        = null;
  protected ScaleInterpolator myGrowInterpolator = null;

  public ES_DesolokChild(Transform3D t, ES_Window w, ES_Desolok mother)
  {
    super(t,w);
    myFlags.set(ES.BononafChild);

    myMother = mother;

    TransformGroup myNewBody = myWindow.myModelCache.getModel("DesolokChild");

    TransformGroup myGrowTG = new TransformGroup();

    myGrowAlpha = new Alpha(1,Alpha.INCREASING_ENABLE,ES_Util.GetESTime()+1,
       0, CHILDTIME ,
       0, 1,
       0, 0, 0);

    myGrowInterpolator = new ScaleInterpolator(myGrowAlpha,myGrowTG);

    myGrowInterpolator.setSchedulingBounds(ES_Util.GetInfiniteBounds());

    ES_Util.SetESCapabilities(myGrowTG);

    myGrowTG.addChild(myNewBody);

    myBody.insertChild(myGrowTG,ES.BNI_Body);
    myBody.insertChild(myGrowInterpolator, BNI_GROWBEHAVIOR);

    setMyName(myMother.getMyName() + "'s child" );

   // homeBranch.compile();
    myWindow.objRoot.addChild(homeBranch);

    myWindow.numDesoloks++;

  }

  public void FrameThink()
  {
    super.FrameThink();

    if (myMoves.QueueIsEmpty())
    {
      if      (curState == STATE_WANDER)
      {
        if (ES_Util.GetRndBool())
          myMoves.EnqueueMove(new ES_Move(this,myMoves,MOVESPEED, LOOKSPEED,new Point3f(ES_Util.GetRndFloat(-0.5f,0.5f) , 0, ES_Util.GetRndFloat(-0.5f,0.5f) )));
        else
          myMoves.EnqueueMove(new ES_Move(this,myMoves,MOVESPEED, LOOKSPEED,new Point3f(ES_Util.GetRndFloat(-0.25f,0.25f) , 0, ES_Util.GetRndFloat(0.3f,0.6f) )));

        myMoves.SetMoveMode(ES.MM_MoveASAP);
        setFlag(ES.isMoving);
        return;
      }
      else if (curState == STATE_ENROUTETOPLACE)
      {
        curState = STATE_WANDER;
        return;
      }
      else if (curState == STATE_ENROUTETOFOOD)
      {
        curState = STATE_EATFOOD;
        return;
      }
    }

  }

  public void Think()
  {
    super.Think();
    myThink.setNextThink((int)ES_Util.GetRndLong(900,1100));

    if (ES_Util.GetESTime() > timeborn+CHILDTIME)
    {
      GrowUp();
      return;
    }

    if (curState == STATE_EATFOOD)
    {
      Eat();
      return;
    }

    Hunger();

  }

  public void GrowUp()
  {
    myWindow.numDesoloks--;
    ES_Desolok grownUp = new ES_Desolok(GetTransform(),myWindow);
    homeBranch.detach();
  }

  public void Hunger()
  {
    hunger+=ES_Util.GetRndFloat()+0.4f;

    if (hunger > DECIDE_EAT && curState == STATE_WANDER)
    {
        SearchFood();
    }
  }


  public void SearchFood()
  {
    wantToEat = ES_Util.GetClosestLifeFormInCone(myWindow.objRoot,GetTransform(),LOOKDISTANCE,LOOKANGLE,ES.Flower);

    if (wantToEat == null && hunger > CRITICAL_EAT - 25.0f && lastKnownFlowerPos != null)
    {
       ES_Util.BuildRoute(this,new Transform3D(lastKnownFlowerPos),0.12f,MOVESPEED*2.0f,LOOKSPEED);
       lastKnownFlowerPos = null;
       curState = STATE_ENROUTETOPLACE;
       return;
    }

    if (wantToEat != null)
    {
       ES_Util.BuildRoute(this,wantToEat.GetTransform(),0.12f,MOVESPEED*2.0f,LOOKSPEED);
       curState = STATE_ENROUTETOFOOD;
    }
  }

  public void Eat()
  {
    if (wantToEat == null)
    {
      curState = STATE_WANDER;
      return;
    }

    if (!wantToEat.homeBranch.isLive())
    {
      curState = STATE_WANDER;
      return;
    }

    float lesshunger = -1.0f;
    if (wantToEat.hasFlag(ES.Flower))
    {
      lastKnownFlowerPos = new Transform3D(wantToEat.GetTransform());

      float flowergrowth = ((ES_Flower)wantToEat).growth;

      final float oldgrowth = flowergrowth;
      flowergrowth -= hunger;
      lesshunger     = Math.min(hunger,oldgrowth);
      hunger       -= lesshunger;

      ((ES_Flower)wantToEat).setGrowth(flowergrowth);
    }
    curState = STATE_WANDER;
  }

  public void Die()
  {
    myWindow.numDesoloks--;
    ES_DesolokCorpse myCorpse = new ES_DesolokCorpse(getMyName(), (-1), GetTransform(), myWindow);
    homeBranch.detach();
  }

  public void CheckCriticals()
  {
    if (hunger > CRITICAL_EAT)
      Die();
  }

  public String curStateToString()
  {
    final StringBuffer retBuf = new StringBuffer();

    if (curState == STATE_WANDER)
      retBuf.append("Wandering");
    else if (curState == STATE_ENROUTETOPLACE)
      retBuf.append("En Route To Place");
    else if (curState == STATE_SEARCHFOOD)
      retBuf.append("Searching Food");
    else if (curState == STATE_ENROUTETOFOOD)
      retBuf.append("En Route To Food");
    else if (curState == STATE_EATFOOD)
      retBuf.append("Eating Food");

    return new String(retBuf);

  }
}