/******************************************************************************

ES_Move
-------------------------------------------------------------------------------
Encapsulates one Move from the current location to an offset of it
and make sure the animal looks in the direction its walking

-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Feb 8, 2002
Version:    1.0

******************************************************************************/

package EcoSystem;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.lang.Double;
import java.util.*;


public class ES_Move
{
  protected ES_MoveQueue               myMoveQueue    = null;
  protected ES_Animal                  myAnimal       = null;
  protected Alpha                      myMoveAlpha    = null;
  protected Alpha                      myLookAlpha    = null;
  protected ES_LookBehavior            myLookBehavior = null;
  protected ES_MoveBehavior            myMoveBehavior = null;
  protected Point3f                    myPosition     = null;
  protected Point3f                    myGoal         = null;
  protected float                      myMoveSpeed    = 0.0f;
  protected float                      myLookSpeed    = 0.0f;
  protected boolean                    isMoving       = false;
  protected boolean                    isAborted      = false;
  protected long                       myFinishTime   = Long.MAX_VALUE;

    public ES_Move(ES_Animal animal, ES_MoveQueue movequeue, float movespeed, float lookspeed, Point3f goal)
    {
      myAnimal = animal;
      myGoal = goal;
      myMoveSpeed = movespeed;
      myLookSpeed = lookspeed;
      myMoveQueue = movequeue;

      myPosition = new Point3f(0,0,0);
    }

    public boolean isMoveValid()
    {
      final Point3f checkGoal = new Point3f(myGoal);
      myAnimal.GetTransform().transform(checkGoal);
      return ES_Util.IsInWorld(checkGoal);
    }

    public boolean isMoveFinished()
    {
      if (isAborted)
        return true;


      return (ES_Util.GetESTime() > myFinishTime);
/*      if (myMoveAlpha != null)
      {
        return myMoveAlpha.finished();
      }
      else
      {
        return false;
      }*/
    }

    public boolean isMoving()
    {
      return isMoving;
    }

    public void startMove()
    {
      isMoving = true;
      final float[]  knots = {0.0f,1.0f};


      final Transform3D mytrans   = new Transform3D();
      final Transform3D rottrans  = new Transform3D();
      final Vector3d    nowvec   = new Vector3d();
      final Vector3d    movevec  = new Vector3d();
      final Quat4f      nowrot   = new Quat4f();
      final Quat4f      moverot  = new Quat4f();
      float       tmpangle = 0.0f;

      myAnimal.myBody.getTransform(mytrans);

      mytrans.get(nowrot);

      nowvec.set((Tuple3f)new Vector3f(0,0,1));

      movevec.set((Tuple3f)myGoal);
      movevec.normalize();

      if (ES_Util.IsAnglePositive(nowvec,movevec))
      {
        tmpangle = (float)nowvec.angle(movevec);//+
      }
      else
      {
        tmpangle = -1 * (float)nowvec.angle(movevec);//-
      }

      rottrans.rotY(tmpangle);
      rottrans.get(moverot);
      moverot.mul(nowrot);

      mytrans.transform(myPosition);
      mytrans.transform(myGoal);

      final Quat4f[] rotations = {nowrot,moverot};
      final Point3f[] rotpositions = {myPosition,myPosition};

      final Quat4f[] posrotations = {moverot,moverot};
      final Point3f[] positions = {myPosition, myGoal};

//       ES_Util.DebugPrintLn("constructing alphas @ ");

       final long startmove = ES_Util.GetESTime()+250;
       final long looktime  = (long)Math.round((Math.toDegrees(nowvec.angle(movevec))/myLookSpeed)*1000);
       final long movetime  = (long)Math.round((myPosition.distance(myGoal)/myMoveSpeed)*1000);

       myLookAlpha = new Alpha(1,Alpha.INCREASING_ENABLE,startmove,0, looktime , 0, 1, 0, 0, 0);

       myMoveAlpha = new Alpha(1,Alpha.INCREASING_ENABLE, startmove+looktime+1, 0, movetime ,0, 1,0, 0, 0);

//      ES_Util.DebugPrintLn("move will be finished:"+(ES_Util.GetESTime()+1+(long)Math.round((Math.toDegrees(nowvec.angle(movevec))/myLookSpeed)*1000.0f+(long)Math.round((myPosition.distance(myGoal)/myMoveSpeed)*1000.0f)))+" @ ");

      myFinishTime = startmove+looktime+1+movetime+1;

      final BranchGroup b = new BranchGroup();
      ES_Util.SetESCapabilities(b);

      myMoveBehavior = new ES_MoveBehavior(myAnimal,b,myMoveAlpha,knots,posrotations,positions);
      myLookBehavior = new ES_LookBehavior(myAnimal,myLookAlpha,knots,rotations,rotpositions);

      b.addChild(myMoveBehavior);
      b.addChild(myLookBehavior);

      myAnimal.myBody.addChild(b);

      Point3d trailStart = new Point3d(myPosition);
      Point3d trailEnd   = new Point3d(myGoal);
      //ES_Util.PrintTrail(myAnimal.homeBranch, trailStart,trailEnd );
//*/
    }

    public void abortMove()
    {
//      ES_Util.DebugPrintLn("Aborting move @ ");
      if (myMoveBehavior != null)
        myMoveBehavior.abortMove();
      isMoving = false;
      isAborted = true;

    }

    public void performMove()
    {
      if (isMoveFinished())
      {
//        ES_Util.DebugPrintLn("move finished @ ");
        isMoving = false;
      }

    }

    public void printMove()
    {

    }

}